//+------------------------------------------------------------------+
//|                                                  StepMA_v7.1.mq4 |
//|                                Copyright ?2006, TrendLaboratory |
//|            http://finance.groups.yahoo.com/group/TrendLaboratory |
//|                                   E-mail: igorad2003@yahoo.co.uk |
//+------------------------------------------------------------------+
//| Modifications:                                                   |
//| - StepMA_v7.2, 2006/06/26, by Ogeima, ph_bresson@yahoo.com       |
//|   - Color Mode 3                                                 |
//|   - UpDownShift to ColorMode2 and ColorMode3                     |
//|   - Fixed LineBuffer[] for every ColorMode, in order to enable   |
//|     applying a MovingAverage(1) on top of the StepMA,            | 
//|     thus enabling the charting of LevelLines on the StepMA       |
//+------------------------------------------------------------------+

#property copyright "Copyright ?2006, TrendLaboratory"
#property link      "http://finance.groups.yahoo.com/group/TrendLaboratory"

#property indicator_chart_window
#property indicator_buffers 7
#property indicator_color1 Red
#property indicator_color2 MediumSeaGreen
#property indicator_color3 Red
#property indicator_color4 MediumSeaGreen
#property indicator_color5 Aqua
#property indicator_color6 LimeGreen
#property indicator_color7 Orange

#property indicator_width1 1
#property indicator_width2 1
#property indicator_width3 3
#property indicator_width4 3
#property indicator_width5 2
#property indicator_width6 2
#property indicator_width7 2

//---- input parameters
extern bool ShowHeiken_Ashi_On    = TRUE;
extern ENUM_MA_METHOD haMaMetod   = MODE_SMMA;
extern int            haMaPeriod  = 6;
extern ENUM_MA_METHOD haMaMetod2  = MODE_LWMA;
extern int            haMaPeriod2 = 2;
extern color          haBullColor = MediumSeaGreen;
extern color          haBearColor = Red;
extern string  ______      = "set StepMA";
extern bool StepMaForHA_On = TRUE;
extern int     Length      = 1;      // Volty Length
extern double  Kv          = 5;       // Sensivity Factor
extern int     StepSize    = 2;      // Constant Step Size (if need)
extern double  Variance    = 1;
extern double  Mean        = 0.0;
extern int     MA_Mode     = 0;       // Volty MA Mode : 0-SMA, 1-LWMA 
extern int     Advance     = 0;       // Offset
extern double  Percentage  = 0;       // Percentage of Up/Down Moving
extern double  UpDownShift = 0;       // Number of Up/Down Moving, in points. For ColorMode==3 only.
extern bool    HighLow     = false;   // High/Low Mode Switch (more sensitive)
extern int     ColorMode   = 2;       // Color Mode Switch
extern int     BarsNumber  = 0;       
extern bool   alertsOn           = false;
extern bool   alertsOnCurrent    = true;
extern bool   alertsMessage      = true;
extern bool   alertsNotification = false;
extern bool   alertsSound        = false;
extern bool   alertsEmail        = false;

//---- indicator buffers
double LineBuffer[];
double UpBuffer[];
double DnBuffer[];
double smin[];
double smax[];
double trend[];
double _coeffs[];

double ExtMapBuffer1[];
double ExtMapBuffer2[];
double ExtMapBuffer3[];
double ExtMapBuffer4[];
double ExtMapBuffer5[];
double ExtMapBuffer6[];
double ExtMapBuffer7[];
double ExtMapBuffer8[];

double StepMA=0, ATR0=0,ATRmax=-100000,ATRmin=1000000;
int limit;

//---- StepSize Calculation

   double StepSizeCalc ( int Len, double Km, int Size, int k)
   {

   double result;
   if( Size==0 ) 
   {
        double AvgRange=0;
	     double Weight = 0;
	     for (int i=Len-1;i>=0;i--)
	     { 
            if(MA_Mode==0) double alfa= 1.0; else alfa= 1.0*(Len-i)/Len;
            double smpHigh=High[k+i];
            double smpLow =Low[k+i];
            if(StepMaForHA_On)
            {
             smpHigh=MathMax(ExtMapBuffer1[k+i],ExtMapBuffer2[k+i]);
             smpLow =MathMin(ExtMapBuffer1[k+i],ExtMapBuffer2[k+i]);
            }
            AvgRange+= alfa*(smpHigh-smpLow);
            Weight += alfa;
        }
	     ATR0 = AvgRange/Weight;
	
	if (ATR0>ATRmax) ATRmax=ATR0;
	if (ATR0<ATRmin) ATRmin=ATR0;
	
	result=MathRound(0.5*Km*(ATRmax+ATRmin)/Point); 
	}
	else
	result=Km*StepSize;
   
   return(result);
   }

//---- StepMA Calculation   
   
   double StepMACalc (int tLength, bool HL, double Size, int k)
   {
   int counted_bars=IndicatorCounted();
   double result;
   double smpHigh=High[k];
   double smpLow =Low[k];
   double smpClose=Close[k];
   if(StepMaForHA_On)
    {
     smpHigh=MathMax(ExtMapBuffer1[k],ExtMapBuffer2[k]);
     smpLow =MathMin(ExtMapBuffer1[k],ExtMapBuffer2[k]);
     smpClose=ExtMapBuffer4[k];
    }
   
   if (HL)
	  {
	  smax[k]=iPdfma(smpLow ,tLength,_coeffs,k,0)+2.0*Size*Point;
	  smin[k]=iPdfma(smpHigh,tLength,_coeffs,k,1)-2.0*Size*Point;
     } 	
	else  	 
	  {
	  smax[k]=iPdfma(smpClose,tLength,_coeffs,k,0)+2.0*Size*Point;
	  smin[k]=iPdfma(smpClose,tLength,_coeffs,k,0)-2.0*Size*Point;
	  }
	  if (counted_bars==0){smax[limit+1]=smax[limit];smin[limit+1]=smin[limit];trend[limit+1]=0;}
	  
	  trend[k]=trend[k+1];   
	  
	  if (smpClose>smax[k+1]) trend[k]=1;   
	 
	  if (smpClose<smin[k+1]) trend[k]=-1;  
	 
	  if(trend[k]>0)
	  {
	  if(smin[k]<smin[k+1]) smin[k]=smin[k+1];
	  result=smin[k]+Size*Point;
	  } 
	  else
	  {
	  if(smax[k]>smax[k+1]) smax[k]=smax[k+1];
	  result=smax[k]-Size*Point;
	  }
     //Print (" k=",k," trend=",trend[k], " res=",result," Smax=", smax[k], " Smin=", smin[k]);
	  
     
     return(result); 
     } 
//+------------------------------------------------------------------+
//| Custom indicator initialization function                         |
//+------------------------------------------------------------------+
#define maxx 3.5
  int init()
  {
   string short_name;
//---- indicator line
   IndicatorBuffers(14);
   if(ShowHeiken_Ashi_On == TRUE)
   {
   SetIndexStyle(0,DRAW_HISTOGRAM, STYLE_SOLID,EMPTY,haBearColor);
   SetIndexStyle(1,DRAW_HISTOGRAM, STYLE_SOLID,EMPTY,haBullColor);
   SetIndexStyle(2,DRAW_HISTOGRAM, STYLE_SOLID,EMPTY,haBearColor);
   SetIndexStyle(3,DRAW_HISTOGRAM, STYLE_SOLID,EMPTY,haBullColor);
   }
   else
   {
   SetIndexStyle(0,DRAW_NONE);
   SetIndexStyle(1,DRAW_NONE);
   SetIndexStyle(2,DRAW_NONE);
   SetIndexStyle(3,DRAW_NONE);
   }
   
   SetIndexBuffer(0,ExtMapBuffer1);
   SetIndexBuffer(1,ExtMapBuffer2);
   SetIndexBuffer(2,ExtMapBuffer3);
   SetIndexBuffer(3,ExtMapBuffer4);
   
   SetIndexBuffer(10,ExtMapBuffer5);
   SetIndexBuffer(11,ExtMapBuffer6);
   SetIndexBuffer(12,ExtMapBuffer7);
   SetIndexBuffer(13,ExtMapBuffer8);
   
//   SetIndexStyle(0,DRAW_LINE,STYLE_SOLID,2);
   SetIndexStyle(4,DRAW_LINE);
   SetIndexStyle(5,DRAW_LINE);
   SetIndexStyle(6,DRAW_LINE);
   
   SetIndexArrow(5,159);
   SetIndexArrow(6,159);
   
   SetIndexBuffer(4,LineBuffer);
   SetIndexBuffer(5,UpBuffer);
   SetIndexBuffer(6,DnBuffer);
   
   SetIndexShift(4,Advance);
   SetIndexShift(5,Advance);
   SetIndexShift(6,Advance);
   
   SetIndexBuffer(7,smin);
   SetIndexBuffer(8,smax);
   SetIndexBuffer(9,trend);
   
//---- name for DataWindow and indicator subwindow label
   short_name="StepMA("+Length+","+Kv+","+StepSize+")";
   IndicatorShortName(short_name);
   SetIndexLabel(0,short_name);
   SetIndexLabel(1,"UpTrend");
   SetIndexLabel(2,"DownTrend"); 
//----
   SetIndexDrawBegin(0,Length);
   SetIndexDrawBegin(1,Length);
   SetIndexDrawBegin(2,Length);
//----

      ArrayResize(_coeffs,Length);   
         double step = maxx/(Length-1); for(int i=0; i<Length; i++) _coeffs[i] = pdf(i*step,Variance,Mean*maxx);

   return(0);
   }
//+------------------------------------------------------------------+
//| Custor indicator deinitialization function                       |
//+------------------------------------------------------------------+
int deinit()
  {
//---- 
   
//----
   Comment("");
   return(0);
  }     
//+------------------------------------------------------------------+
//| StepMA_v7.2                                                        |
//+------------------------------------------------------------------+
int start()
{
   int shift, counted_bars=IndicatorCounted();
     
   if ( BarsNumber > 0 ) int Nbars=BarsNumber; else Nbars=Bars;
   if ( counted_bars > 0 )  limit=Bars-counted_bars;
   if ( counted_bars < 0 )  return(0);
   if ( counted_bars ==0 )  limit=Nbars-Length-1;
   
   int pos=Bars-counted_bars-1;
   int pos2=pos;
   double maOpen, maClose, maLow, maHigh;
   double haOpen, haHigh, haLow, haClose;
   while(pos>=0)
     {
      maOpen  = iMA(NULL,0,haMaPeriod,0,haMaMetod,PRICE_OPEN ,pos);
      maClose = iMA(NULL,0,haMaPeriod,0,haMaMetod,PRICE_CLOSE,pos);
      maLow   = iMA(NULL,0,haMaPeriod,0,haMaMetod,PRICE_LOW  ,pos);
      maHigh  = iMA(NULL,0,haMaPeriod,0,haMaMetod,PRICE_HIGH ,pos);
      
      haOpen=(ExtMapBuffer5[pos+1]+ExtMapBuffer6[pos+1])/2;
      haClose=(maOpen+maHigh+maLow+maClose)/4;
      haHigh=MathMax(maHigh, MathMax(haOpen, haClose));
      haLow=MathMin(maLow, MathMin(haOpen, haClose));

      if (haOpen<haClose) 
        {
         ExtMapBuffer7[pos]=haLow;
         ExtMapBuffer8[pos]=haHigh;
        } 
      else
        {
         ExtMapBuffer7[pos]=haHigh;
         ExtMapBuffer8[pos]=haLow;
        } 
      ExtMapBuffer5[pos]=haOpen;
      ExtMapBuffer6[pos]=haClose;

 	   pos--;
     }
     
   int i;
   for(i=0; i<pos2; i++) ExtMapBuffer1[i]=iMAOnArray(ExtMapBuffer7,Bars,haMaPeriod2,0,haMaMetod2,i);
   for(i=0; i<pos2; i++) ExtMapBuffer2[i]=iMAOnArray(ExtMapBuffer8,Bars,haMaPeriod2,0,haMaMetod2,i);
   for(i=0; i<pos2; i++) ExtMapBuffer3[i]=iMAOnArray(ExtMapBuffer5,Bars,haMaPeriod2,0,haMaMetod2,i);
   for(i=0; i<pos2; i++) ExtMapBuffer4[i]=iMAOnArray(ExtMapBuffer6,Bars,haMaPeriod2,0,haMaMetod2,i);
   
   
   
   
     
   for(shift=limit;shift>=0;shift--) 
   {	
     
      int Step = StepSizeCalc( Length, Kv, StepSize, shift);
				
      Comment (" StepSize= ", Step);
	
   	StepMA = StepMACalc ( Length, HighLow, Step, shift)+Percentage/100.0*Step*Point;
	
   	if ( ColorMode == 0) LineBuffer[shift]=StepMA;
	
   	if ( ColorMode == 1)
   	{
         if ( trend[shift]>0 )
         {
            UpBuffer[shift]=StepMA-Step*Point;
            LineBuffer[shift]= UpBuffer[shift];
            DnBuffer[shift]=EMPTY_VALUE;
         }
         else
         if ( trend[shift]<0 )
         {
            DnBuffer[shift]=StepMA+Step*Point;
            LineBuffer[shift]= DnBuffer[shift];
            UpBuffer[shift]=EMPTY_VALUE;
         }
      }
      else
      if ( ColorMode == 2)
      {
         if (trend[shift]>0) 
         {
            UpBuffer[shift]=StepMA+UpDownShift*Point;
            if ( trend[shift+1] < 0 ) UpBuffer[shift+1] = DnBuffer[shift+1];
            LineBuffer[shift]= UpBuffer[shift];
            DnBuffer[shift]=EMPTY_VALUE;
         }
         if (trend[shift]<0) 
         {
            DnBuffer[shift]=StepMA-UpDownShift*Point;
            if ( trend[shift+1] > 0 ) DnBuffer[shift+1] = UpBuffer[shift+1];
            LineBuffer[shift]= DnBuffer[shift];
            UpBuffer[shift]=EMPTY_VALUE;
         }
      }//end if ( ColorMode == 2)
      else
      if ( ColorMode == 3)
   	{
         if ( trend[shift]>0 )
         {
            UpBuffer[shift]=StepMA+(Step+UpDownShift)*Point;
            LineBuffer[shift]= UpBuffer[shift];
            DnBuffer[shift]=EMPTY_VALUE;
         }
         else
         if ( trend[shift]<0 )
         {
            DnBuffer[shift]=StepMA-(Step+UpDownShift)*Point;
            LineBuffer[shift]= DnBuffer[shift];
            UpBuffer[shift]=EMPTY_VALUE;
         }
      }//end if ( ColorMode == 3)
      else
      {	
         LineBuffer[shift]=StepMA;
         UpBuffer[shift]=EMPTY_VALUE;
         DnBuffer[shift]=EMPTY_VALUE;
      }
	
   }//end for(shift=limit;shift>=0;shift--) 
   
         if (alertsOn)
         {
            if (alertsOnCurrent)
                 int whichBar = 0;
            else     whichBar = 1;
            if (trend[whichBar] != trend[whichBar+1])
            if (trend[whichBar] == 1)
                  doAlert("up");
            else  doAlert("down");       
         }   
	return(0);	
 }

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//
//
//

void doAlert(string doWhat)
{
   static string   previousAlert="nothing";
   static datetime previousTime;
   string message;
   
      if (previousAlert != doWhat || previousTime != Time[0]) {
          previousAlert  = doWhat;
          previousTime   = Time[0];

          //
          //
          //
          //
          //

          message =  Symbol()+" at "+TimeToStr(TimeLocal(),TIME_SECONDS)+" StepMa pdf changed trend to "+doWhat;
             if (alertsMessage)      Alert(message);
             if (alertsNotification) SendNotification(message);
             if (alertsEmail)        SendMail(StringConcatenate(Symbol()," StepMa pdf "),message);
             if (alertsSound)        PlaySound("alert2.wav");
      }
}

//------------------------------------------------------------------
//                                                                  
//------------------------------------------------------------------
//
//
//    normal probability density function
//
//

#define Pi 3.141592653589793238462643

double pdf(double x, double variance=1.0, double mean=0)
      { return((1.0/MathSqrt(2*Pi*MathPow(variance,2))*MathExp(-MathPow(x-mean,2)/(2*MathPow(variance,2))))); }

//
//
//
//
//

double workPdfma[][2];
double iPdfma(double price, double period, double& coeffs[], int r, int instanceNo=0)
{
   if (ArraySize(workPdfma)!= Bars) ArrayResize(workPdfma,Bars); r = Bars-r-1;
   
   //
   //
   //
   //
   //
   
   workPdfma[r][instanceNo] = price;
      double sumw = coeffs[0];
      double sum  = coeffs[0]*price;

      for(int k=1; k<period && (r-k)>=0; k++)
      {
         double weight = coeffs[k];
                sumw  += weight;
                sum   += weight*workPdfma[r-k][instanceNo];  
      }             
      return(sum/sumw);
}
